package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickHATZero;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickHATZeroActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickHATZeroReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickHATZeroWrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickHATZeroReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickHATZeroGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATZeroWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickHATZeroGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickHATZeroGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATZeroWrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickHATZeroGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickHATZeroGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATZeroWrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickHATZeroGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickHATZeroGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHATZero.SPITFPErrorCount value = ((BrickHATZeroWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickHATZeroGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickHATZeroGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHATZero.Identity value = ((BrickHATZeroWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickHATZeroGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATZeroActions) {
            return ((BrickHATZeroActions) actions).brickHATZeroGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATZeroActions class.");
        }
    }
}
