package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickSilentStepper;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickSilentStepperActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Max Velocity")
    public void brickSilentStepperSetMaxVelocity(
            @ActionInput(name = "velocity") int velocity) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setMaxVelocity(velocity);
    }

    public static void brickSilentStepperSetMaxVelocity(@Nullable ThingActions actions, int velocity) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetMaxVelocity(velocity);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Max Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> brickSilentStepperGetMaxVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getMaxVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetMaxVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetMaxVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Current Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> brickSilentStepperGetCurrentVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getCurrentVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetCurrentVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetCurrentVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Speed Ramping")
    public void brickSilentStepperSetSpeedRamping(
            @ActionInput(name = "acceleration") int acceleration,
            @ActionInput(name = "deacceleration") int deacceleration) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setSpeedRamping(acceleration, deacceleration);
    }

    public static void brickSilentStepperSetSpeedRamping(@Nullable ThingActions actions, int acceleration, int deacceleration) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetSpeedRamping(acceleration, deacceleration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Speed Ramping")
    public @ActionOutput(name = "acceleration", type="int")
           @ActionOutput(name = "deacceleration", type="int")
           Map<String, Object> brickSilentStepperGetSpeedRamping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.SpeedRamping value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSpeedRamping();
        result.put("acceleration", value.acceleration);
        result.put("deacceleration", value.deacceleration);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSpeedRamping(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSpeedRamping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Full Brake")
    public void brickSilentStepperFullBrake(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).fullBrake();
    }

    public static void brickSilentStepperFullBrake(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperFullBrake();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Steps")
    public void brickSilentStepperSetSteps(
            @ActionInput(name = "steps") int steps) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setSteps(steps);
    }

    public static void brickSilentStepperSetSteps(@Nullable ThingActions actions, int steps) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetSteps(steps);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Steps")
    public @ActionOutput(name = "steps", type="int")
           Map<String, Object> brickSilentStepperGetSteps(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSteps();
        result.put("steps", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSteps(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSteps();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Remaining Steps")
    public @ActionOutput(name = "steps", type="int")
           Map<String, Object> brickSilentStepperGetRemainingSteps(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getRemainingSteps();
        result.put("steps", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetRemainingSteps(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetRemainingSteps();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Drive Forward")
    public void brickSilentStepperDriveForward(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).driveForward();
    }

    public static void brickSilentStepperDriveForward(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperDriveForward();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Drive Backward")
    public void brickSilentStepperDriveBackward(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).driveBackward();
    }

    public static void brickSilentStepperDriveBackward(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperDriveBackward();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Stop")
    public void brickSilentStepperStop(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).stop();
    }

    public static void brickSilentStepperStop(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperStop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Motor Current")
    public void brickSilentStepperSetMotorCurrent(
            @ActionInput(name = "current") int current) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setMotorCurrent(current);
    }

    public static void brickSilentStepperSetMotorCurrent(@Nullable ThingActions actions, int current) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetMotorCurrent(current);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Motor Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickSilentStepperGetMotorCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getMotorCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetMotorCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetMotorCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Enable")
    public void brickSilentStepperEnable(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).enable();
    }

    public static void brickSilentStepperEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void brickSilentStepperDisable(
            ) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).disable();
    }

    public static void brickSilentStepperDisable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperDisable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickSilentStepperIsEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickSilentStepperWrapper)this.handler.getDevice()).isEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperIsEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Basic Configuration")
    public void brickSilentStepperSetBasicConfiguration(
            @ActionInput(name = "standstillCurrent") int standstillCurrent,
            @ActionInput(name = "motorRunCurrent") int motorRunCurrent,
            @ActionInput(name = "standstillDelayTime") int standstillDelayTime,
            @ActionInput(name = "powerDownTime") int powerDownTime,
            @ActionInput(name = "stealthThreshold") int stealthThreshold,
            @ActionInput(name = "coolstepThreshold") int coolstepThreshold,
            @ActionInput(name = "classicThreshold") int classicThreshold,
            @ActionInput(name = "highVelocityChopperMode") boolean highVelocityChopperMode) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setBasicConfiguration(standstillCurrent, motorRunCurrent, standstillDelayTime, powerDownTime, stealthThreshold, coolstepThreshold, classicThreshold, highVelocityChopperMode);
    }

    public static void brickSilentStepperSetBasicConfiguration(@Nullable ThingActions actions, int standstillCurrent, int motorRunCurrent, int standstillDelayTime, int powerDownTime, int stealthThreshold, int coolstepThreshold, int classicThreshold, boolean highVelocityChopperMode) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetBasicConfiguration(standstillCurrent, motorRunCurrent, standstillDelayTime, powerDownTime, stealthThreshold, coolstepThreshold, classicThreshold, highVelocityChopperMode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Basic Configuration")
    public @ActionOutput(name = "standstillCurrent", type="int")
           @ActionOutput(name = "motorRunCurrent", type="int")
           @ActionOutput(name = "standstillDelayTime", type="int")
           @ActionOutput(name = "powerDownTime", type="int")
           @ActionOutput(name = "stealthThreshold", type="int")
           @ActionOutput(name = "coolstepThreshold", type="int")
           @ActionOutput(name = "classicThreshold", type="int")
           @ActionOutput(name = "highVelocityChopperMode", type="boolean")
           Map<String, Object> brickSilentStepperGetBasicConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.BasicConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getBasicConfiguration();
        result.put("standstillCurrent", value.standstillCurrent);
        result.put("motorRunCurrent", value.motorRunCurrent);
        result.put("standstillDelayTime", value.standstillDelayTime);
        result.put("powerDownTime", value.powerDownTime);
        result.put("stealthThreshold", value.stealthThreshold);
        result.put("coolstepThreshold", value.coolstepThreshold);
        result.put("classicThreshold", value.classicThreshold);
        result.put("highVelocityChopperMode", value.highVelocityChopperMode);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetBasicConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetBasicConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Current Position")
    public void brickSilentStepperSetCurrentPosition(
            @ActionInput(name = "position") int position) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setCurrentPosition(position);
    }

    public static void brickSilentStepperSetCurrentPosition(@Nullable ThingActions actions, int position) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetCurrentPosition(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Current Position")
    public @ActionOutput(name = "position", type="int")
           Map<String, Object> brickSilentStepperGetCurrentPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getCurrentPosition();
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetCurrentPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetCurrentPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Target Position")
    public void brickSilentStepperSetTargetPosition(
            @ActionInput(name = "position") int position) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setTargetPosition(position);
    }

    public static void brickSilentStepperSetTargetPosition(@Nullable ThingActions actions, int position) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetTargetPosition(position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Target Position")
    public @ActionOutput(name = "position", type="int")
           Map<String, Object> brickSilentStepperGetTargetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getTargetPosition();
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetTargetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetTargetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Step Configuration")
    public void brickSilentStepperSetStepConfiguration(
            @ActionInput(name = "stepResolution") int stepResolution,
            @ActionInput(name = "interpolation") boolean interpolation) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setStepConfiguration(stepResolution, interpolation);
    }

    public static void brickSilentStepperSetStepConfiguration(@Nullable ThingActions actions, int stepResolution, boolean interpolation) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetStepConfiguration(stepResolution, interpolation);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Step Configuration")
    public @ActionOutput(name = "stepResolution", type="int")
           @ActionOutput(name = "interpolation", type="boolean")
           Map<String, Object> brickSilentStepperGetStepConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.StepConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getStepConfiguration();
        result.put("stepResolution", value.stepResolution);
        result.put("interpolation", value.interpolation);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetStepConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetStepConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Stack Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickSilentStepperGetStackInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getStackInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetStackInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetStackInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get External Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickSilentStepperGetExternalInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getExternalInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetExternalInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetExternalInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Spreadcycle Configuration")
    public void brickSilentStepperSetSpreadcycleConfiguration(
            @ActionInput(name = "slowDecayDuration") int slowDecayDuration,
            @ActionInput(name = "enableRandomSlowDecay") boolean enableRandomSlowDecay,
            @ActionInput(name = "fastDecayDuration") int fastDecayDuration,
            @ActionInput(name = "hysteresisStartValue") int hysteresisStartValue,
            @ActionInput(name = "hysteresisEndValue") int hysteresisEndValue,
            @ActionInput(name = "sineWaveOffset") int sineWaveOffset,
            @ActionInput(name = "chopperMode") int chopperMode,
            @ActionInput(name = "comparatorBlankTime") int comparatorBlankTime,
            @ActionInput(name = "fastDecayWithoutComparator") boolean fastDecayWithoutComparator) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setSpreadcycleConfiguration(slowDecayDuration, enableRandomSlowDecay, fastDecayDuration, hysteresisStartValue, hysteresisEndValue, sineWaveOffset, chopperMode, comparatorBlankTime, fastDecayWithoutComparator);
    }

    public static void brickSilentStepperSetSpreadcycleConfiguration(@Nullable ThingActions actions, int slowDecayDuration, boolean enableRandomSlowDecay, int fastDecayDuration, int hysteresisStartValue, int hysteresisEndValue, int sineWaveOffset, int chopperMode, int comparatorBlankTime, boolean fastDecayWithoutComparator) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetSpreadcycleConfiguration(slowDecayDuration, enableRandomSlowDecay, fastDecayDuration, hysteresisStartValue, hysteresisEndValue, sineWaveOffset, chopperMode, comparatorBlankTime, fastDecayWithoutComparator);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Spreadcycle Configuration")
    public @ActionOutput(name = "slowDecayDuration", type="int")
           @ActionOutput(name = "enableRandomSlowDecay", type="boolean")
           @ActionOutput(name = "fastDecayDuration", type="int")
           @ActionOutput(name = "hysteresisStartValue", type="int")
           @ActionOutput(name = "hysteresisEndValue", type="int")
           @ActionOutput(name = "sineWaveOffset", type="int")
           @ActionOutput(name = "chopperMode", type="int")
           @ActionOutput(name = "comparatorBlankTime", type="int")
           @ActionOutput(name = "fastDecayWithoutComparator", type="boolean")
           Map<String, Object> brickSilentStepperGetSpreadcycleConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.SpreadcycleConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSpreadcycleConfiguration();
        result.put("slowDecayDuration", value.slowDecayDuration);
        result.put("enableRandomSlowDecay", value.enableRandomSlowDecay);
        result.put("fastDecayDuration", value.fastDecayDuration);
        result.put("hysteresisStartValue", value.hysteresisStartValue);
        result.put("hysteresisEndValue", value.hysteresisEndValue);
        result.put("sineWaveOffset", value.sineWaveOffset);
        result.put("chopperMode", value.chopperMode);
        result.put("comparatorBlankTime", value.comparatorBlankTime);
        result.put("fastDecayWithoutComparator", value.fastDecayWithoutComparator);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSpreadcycleConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSpreadcycleConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Stealth Configuration")
    public void brickSilentStepperSetStealthConfiguration(
            @ActionInput(name = "enableStealth") boolean enableStealth,
            @ActionInput(name = "amplitude") int amplitude,
            @ActionInput(name = "gradient") int gradient,
            @ActionInput(name = "enableAutoscale") boolean enableAutoscale,
            @ActionInput(name = "forceSymmetric") boolean forceSymmetric,
            @ActionInput(name = "freewheelMode") int freewheelMode) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setStealthConfiguration(enableStealth, amplitude, gradient, enableAutoscale, forceSymmetric, freewheelMode);
    }

    public static void brickSilentStepperSetStealthConfiguration(@Nullable ThingActions actions, boolean enableStealth, int amplitude, int gradient, boolean enableAutoscale, boolean forceSymmetric, int freewheelMode) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetStealthConfiguration(enableStealth, amplitude, gradient, enableAutoscale, forceSymmetric, freewheelMode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Stealth Configuration")
    public @ActionOutput(name = "enableStealth", type="boolean")
           @ActionOutput(name = "amplitude", type="int")
           @ActionOutput(name = "gradient", type="int")
           @ActionOutput(name = "enableAutoscale", type="boolean")
           @ActionOutput(name = "forceSymmetric", type="boolean")
           @ActionOutput(name = "freewheelMode", type="int")
           Map<String, Object> brickSilentStepperGetStealthConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.StealthConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getStealthConfiguration();
        result.put("enableStealth", value.enableStealth);
        result.put("amplitude", value.amplitude);
        result.put("gradient", value.gradient);
        result.put("enableAutoscale", value.enableAutoscale);
        result.put("forceSymmetric", value.forceSymmetric);
        result.put("freewheelMode", value.freewheelMode);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetStealthConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetStealthConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Coolstep Configuration")
    public void brickSilentStepperSetCoolstepConfiguration(
            @ActionInput(name = "minimumStallguardValue") int minimumStallguardValue,
            @ActionInput(name = "maximumStallguardValue") int maximumStallguardValue,
            @ActionInput(name = "currentUpStepWidth") int currentUpStepWidth,
            @ActionInput(name = "currentDownStepWidth") int currentDownStepWidth,
            @ActionInput(name = "minimumCurrent") int minimumCurrent,
            @ActionInput(name = "stallguardThresholdValue") int stallguardThresholdValue,
            @ActionInput(name = "stallguardMode") int stallguardMode) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setCoolstepConfiguration(minimumStallguardValue, maximumStallguardValue, currentUpStepWidth, currentDownStepWidth, minimumCurrent, stallguardThresholdValue, stallguardMode);
    }

    public static void brickSilentStepperSetCoolstepConfiguration(@Nullable ThingActions actions, int minimumStallguardValue, int maximumStallguardValue, int currentUpStepWidth, int currentDownStepWidth, int minimumCurrent, int stallguardThresholdValue, int stallguardMode) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetCoolstepConfiguration(minimumStallguardValue, maximumStallguardValue, currentUpStepWidth, currentDownStepWidth, minimumCurrent, stallguardThresholdValue, stallguardMode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Coolstep Configuration")
    public @ActionOutput(name = "minimumStallguardValue", type="int")
           @ActionOutput(name = "maximumStallguardValue", type="int")
           @ActionOutput(name = "currentUpStepWidth", type="int")
           @ActionOutput(name = "currentDownStepWidth", type="int")
           @ActionOutput(name = "minimumCurrent", type="int")
           @ActionOutput(name = "stallguardThresholdValue", type="int")
           @ActionOutput(name = "stallguardMode", type="int")
           Map<String, Object> brickSilentStepperGetCoolstepConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.CoolstepConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getCoolstepConfiguration();
        result.put("minimumStallguardValue", value.minimumStallguardValue);
        result.put("maximumStallguardValue", value.maximumStallguardValue);
        result.put("currentUpStepWidth", value.currentUpStepWidth);
        result.put("currentDownStepWidth", value.currentDownStepWidth);
        result.put("minimumCurrent", value.minimumCurrent);
        result.put("stallguardThresholdValue", value.stallguardThresholdValue);
        result.put("stallguardMode", value.stallguardMode);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetCoolstepConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetCoolstepConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Misc Configuration")
    public void brickSilentStepperSetMiscConfiguration(
            @ActionInput(name = "disableShortToGroundProtection") boolean disableShortToGroundProtection,
            @ActionInput(name = "synchronizePhaseFrequency") int synchronizePhaseFrequency) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setMiscConfiguration(disableShortToGroundProtection, synchronizePhaseFrequency);
    }

    public static void brickSilentStepperSetMiscConfiguration(@Nullable ThingActions actions, boolean disableShortToGroundProtection, int synchronizePhaseFrequency) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetMiscConfiguration(disableShortToGroundProtection, synchronizePhaseFrequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Misc Configuration")
    public @ActionOutput(name = "disableShortToGroundProtection", type="boolean")
           @ActionOutput(name = "synchronizePhaseFrequency", type="int")
           Map<String, Object> brickSilentStepperGetMiscConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.MiscConfiguration value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getMiscConfiguration();
        result.put("disableShortToGroundProtection", value.disableShortToGroundProtection);
        result.put("synchronizePhaseFrequency", value.synchronizePhaseFrequency);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetMiscConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetMiscConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Driver Status")
    public @ActionOutput(name = "openLoad", type="int")
           @ActionOutput(name = "shortToGround", type="int")
           @ActionOutput(name = "overTemperature", type="int")
           @ActionOutput(name = "motorStalled", type="boolean")
           @ActionOutput(name = "actualMotorCurrent", type="int")
           @ActionOutput(name = "fullStepActive", type="boolean")
           @ActionOutput(name = "stallguardResult", type="int")
           @ActionOutput(name = "stealthVoltageAmplitude", type="int")
           Map<String, Object> brickSilentStepperGetDriverStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.DriverStatus value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getDriverStatus();
        result.put("openLoad", value.openLoad);
        result.put("shortToGround", value.shortToGround);
        result.put("overTemperature", value.overTemperature);
        result.put("motorStalled", value.motorStalled);
        result.put("actualMotorCurrent", value.actualMotorCurrent);
        result.put("fullStepActive", value.fullStepActive);
        result.put("stallguardResult", value.stallguardResult);
        result.put("stealthVoltageAmplitude", value.stealthVoltageAmplitude);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetDriverStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetDriverStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Set Time Base")
    public void brickSilentStepperSetTimeBase(
            @ActionInput(name = "timeBase") long timeBase) throws TinkerforgeException {
        ((BrickSilentStepperWrapper)this.handler.getDevice()).setTimeBase(timeBase);
    }

    public static void brickSilentStepperSetTimeBase(@Nullable ThingActions actions, long timeBase) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            ((BrickSilentStepperActions) actions).brickSilentStepperSetTimeBase(timeBase);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Time Base")
    public @ActionOutput(name = "timeBase", type="long")
           Map<String, Object> brickSilentStepperGetTimeBase(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getTimeBase();
        result.put("timeBase", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetTimeBase(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetTimeBase();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get All Data")
    public @ActionOutput(name = "currentVelocity", type="int")
           @ActionOutput(name = "currentPosition", type="int")
           @ActionOutput(name = "remainingSteps", type="int")
           @ActionOutput(name = "stackVoltage", type="int")
           @ActionOutput(name = "externalVoltage", type="int")
           @ActionOutput(name = "currentConsumption", type="int")
           Map<String, Object> brickSilentStepperGetAllData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.AllData value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getAllData();
        result.put("currentVelocity", value.currentVelocity);
        result.put("currentPosition", value.currentPosition);
        result.put("remainingSteps", value.remainingSteps);
        result.put("stackVoltage", value.stackVoltage);
        result.put("externalVoltage", value.externalVoltage);
        result.put("currentConsumption", value.currentConsumption);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetAllData(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetAllData();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get All Data Period")
    public @ActionOutput(name = "period", type="long")
           Map<String, Object> brickSilentStepperGetAllDataPeriod(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getAllDataPeriod();
        result.put("period", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetAllDataPeriod(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetAllDataPeriod();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickSilentStepperGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.Identity value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickSilentStepperIsStatusLEDEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickSilentStepperWrapper)this.handler.getDevice()).isStatusLEDEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperIsStatusLEDEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperIsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickSilentStepperGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type="long")
           Map<String, Object> brickSilentStepperGetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") int communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSendTimeoutCount(communicationMethod);
        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSendTimeoutCount(@Nullable ThingActions actions, int communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type="boolean")
           @ActionOutput(name = "minimumDynamicBaudrate", type="long")
           Map<String, Object> brickSilentStepperGetSPITFPBaudrateConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.SPITFPBaudrateConfig value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSPITFPBaudrateConfig();
        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSPITFPBaudrateConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickSilentStepperGetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickSilentStepper.SPITFPErrorCount value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSPITFPErrorCount(brickletPort);
        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type="long")
           Map<String, Object> brickSilentStepperGetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickSilentStepperWrapper)this.handler.getDevice()).getSPITFPBaudrate(brickletPort);
        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickSilentStepperGetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickSilentStepperActions) {
            return ((BrickSilentStepperActions) actions).brickSilentStepperGetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickSilentStepperActions class.");
        }
    }
}
