package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAnalogOutV3;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutV3Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV3GetOutputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getOutputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetOutputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetOutputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV3GetInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletAnalogOutV3ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletAnalogOutV3GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletAnalogOutV3GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletAnalogOutV3GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletAnalogOutV3GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogOutV3.SPITFPErrorCount value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogOutV3GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogOutV3.Identity value = ((BrickletAnalogOutV3Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV3GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV3Actions) {
            return ((BrickletAnalogOutV3Actions) actions).brickletAnalogOutV3GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV3Actions class.");
        }
    }
}
