package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletBarometerV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletBarometerV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletBarometerV2GetAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getAirPressure();
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type="int")
           Map<String, Object> brickletBarometerV2GetAltitude(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getAltitude();
        result.put("altitude", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetAltitude(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletBarometerV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthAirPressure", type="int")
           @ActionOutput(name = "movingAverageLengthTemperature", type="int")
           Map<String, Object> brickletBarometerV2GetMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletBarometerV2.MovingAverageConfiguration value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getMovingAverageConfiguration();
        result.put("movingAverageLengthAirPressure", value.movingAverageLengthAirPressure);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Reference Air Pressure")
    public @ActionOutput(name = "airPressure", type="int")
           Map<String, Object> brickletBarometerV2GetReferenceAirPressure(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getReferenceAirPressure();
        result.put("airPressure", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetReferenceAirPressure(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetReferenceAirPressure();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletBarometerV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletBarometerV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletBarometerV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletBarometerV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletBarometerV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletBarometerV2.SPITFPErrorCount value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletBarometerV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletBarometerV2.Identity value = ((BrickletBarometerV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletBarometerV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletBarometerV2Actions) {
            return ((BrickletBarometerV2Actions) actions).brickletBarometerV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletBarometerV2Actions class.");
        }
    }
}
