package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletColorV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletColorV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Color")
    public @ActionOutput(name = "r", type="int")
           @ActionOutput(name = "g", type="int")
           @ActionOutput(name = "b", type="int")
           @ActionOutput(name = "c", type="int")
           Map<String, Object> brickletColorV2GetColor(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColorV2.Color value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getColor();
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("c", value.c);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetColor(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetColor();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Illuminance")
    public @ActionOutput(name = "illuminance", type="long")
           Map<String, Object> brickletColorV2GetIlluminance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getIlluminance();
        result.put("illuminance", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetIlluminance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetIlluminance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Color Temperature")
    public @ActionOutput(name = "colorTemperature", type="int")
           Map<String, Object> brickletColorV2GetColorTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getColorTemperature();
        result.put("colorTemperature", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetColorTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetColorTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Light")
    public @ActionOutput(name = "enable", type="boolean")
           Map<String, Object> brickletColorV2GetLight(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getLight();
        result.put("enable", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetLight(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetLight();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "gain", type="int")
           @ActionOutput(name = "integrationTime", type="int")
           Map<String, Object> brickletColorV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColorV2.Configuration value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getConfiguration();
        result.put("gain", value.gain);
        result.put("integrationTime", value.integrationTime);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletColorV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletColorV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletColorV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletColorV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletColorV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletColorV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColorV2.SPITFPErrorCount value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletColorV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletColorV2.Identity value = ((BrickletColorV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletColorV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletColorV2Actions) {
            return ((BrickletColorV2Actions) actions).brickletColorV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletColorV2Actions class.");
        }
    }
}
