package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletCurrent12;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCurrent12Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletCurrent12GetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCurrent12Wrapper)this.handler.getDevice()).getCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent12GetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCurrent12Actions) {
            return ((BrickletCurrent12Actions) actions).brickletCurrent12GetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCurrent12Actions class.");
        }
    }

    @RuleAction(label = "Is Over Current")
    public @ActionOutput(name = "over", type="boolean")
           Map<String, Object> brickletCurrent12IsOverCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletCurrent12Wrapper)this.handler.getDevice()).isOverCurrent();
        result.put("over", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent12IsOverCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCurrent12Actions) {
            return ((BrickletCurrent12Actions) actions).brickletCurrent12IsOverCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCurrent12Actions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletCurrent12GetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCurrent12Wrapper)this.handler.getDevice()).getAnalogValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent12GetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCurrent12Actions) {
            return ((BrickletCurrent12Actions) actions).brickletCurrent12GetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCurrent12Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletCurrent12GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCurrent12.Identity value = ((BrickletCurrent12Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCurrent12GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCurrent12Actions) {
            return ((BrickletCurrent12Actions) actions).brickletCurrent12GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCurrent12Actions class.");
        }
    }
}
