package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDistanceIRV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDistanceIRV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type="int")
           Map<String, Object> brickletDistanceIRV2GetDistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getDistance();
        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetDistance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "analogValue", type="long")
           Map<String, Object> brickletDistanceIRV2GetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getAnalogValue();
        result.put("analogValue", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLength", type="int")
           Map<String, Object> brickletDistanceIRV2GetMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getMovingAverageConfiguration();
        result.put("movingAverageLength", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Distance LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDistanceIRV2GetDistanceLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getDistanceLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetDistanceLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetDistanceLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensor Type")
    public @ActionOutput(name = "sensor", type="int")
           Map<String, Object> brickletDistanceIRV2GetSensorType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getSensorType();
        result.put("sensor", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetSensorType(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetSensorType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletDistanceIRV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletDistanceIRV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDistanceIRV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletDistanceIRV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletDistanceIRV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDistanceIRV2.SPITFPErrorCount value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDistanceIRV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDistanceIRV2.Identity value = ((BrickletDistanceIRV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDistanceIRV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).brickletDistanceIRV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }
}
