package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDustDetector;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDustDetectorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Dust Density")
    public @ActionOutput(name = "dustDensity", type="int")
           Map<String, Object> brickletDustDetectorGetDustDensity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDustDetectorWrapper)this.handler.getDevice()).getDustDensity();
        result.put("dustDensity", value);
        return result;
    }

    public static Map<String, Object> brickletDustDetectorGetDustDensity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDustDetectorActions) {
            return ((BrickletDustDetectorActions) actions).brickletDustDetectorGetDustDensity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDustDetectorActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="int")
           Map<String, Object> brickletDustDetectorGetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDustDetectorWrapper)this.handler.getDevice()).getMovingAverage();
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletDustDetectorGetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDustDetectorActions) {
            return ((BrickletDustDetectorActions) actions).brickletDustDetectorGetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDustDetectorActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDustDetectorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDustDetector.Identity value = ((BrickletDustDetectorWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDustDetectorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDustDetectorActions) {
            return ((BrickletDustDetectorActions) actions).brickletDustDetectorGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDustDetectorActions class.");
        }
    }
}
