package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletGPSV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletGPSV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Coordinates")
    public @ActionOutput(name = "latitude", type="long")
           @ActionOutput(name = "ns", type="char")
           @ActionOutput(name = "longitude", type="long")
           @ActionOutput(name = "ew", type="char")
           Map<String, Object> brickletGPSV2GetCoordinates(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.Coordinates value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getCoordinates();
        result.put("latitude", value.latitude);
        result.put("ns", value.ns);
        result.put("longitude", value.longitude);
        result.put("ew", value.ew);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetCoordinates(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetCoordinates();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status")
    public @ActionOutput(name = "hasFix", type="boolean")
           @ActionOutput(name = "satellitesView", type="int")
           Map<String, Object> brickletGPSV2GetStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.Status value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getStatus();
        result.put("hasFix", value.hasFix);
        result.put("satellitesView", value.satellitesView);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type="int")
           @ActionOutput(name = "geoidalSeparation", type="int")
           Map<String, Object> brickletGPSV2GetAltitude(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.Altitude value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getAltitude();
        result.put("altitude", value.altitude);
        result.put("geoidalSeparation", value.geoidalSeparation);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetAltitude(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetAltitude();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Motion")
    public @ActionOutput(name = "course", type="long")
           @ActionOutput(name = "speed", type="long")
           Map<String, Object> brickletGPSV2GetMotion(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.Motion value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getMotion();
        result.put("course", value.course);
        result.put("speed", value.speed);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetMotion(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetMotion();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "date", type="long")
           @ActionOutput(name = "time", type="long")
           Map<String, Object> brickletGPSV2GetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.DateTime value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getDateTime();
        result.put("date", value.date);
        result.put("time", value.time);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetDateTime();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Satellite System Status")
    public @ActionOutput(name = "satelliteNumbers", type="int[]")
           @ActionOutput(name = "fix", type="int")
           @ActionOutput(name = "pdop", type="int")
           @ActionOutput(name = "hdop", type="int")
           @ActionOutput(name = "vdop", type="int")
           Map<String, Object> brickletGPSV2GetSatelliteSystemStatus(
            @ActionInput(name = "satelliteSystem") int satelliteSystem) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.SatelliteSystemStatus value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getSatelliteSystemStatus(satelliteSystem);
        result.put("satelliteNumbers", value.satelliteNumbers);
        result.put("fix", value.fix);
        result.put("pdop", value.pdop);
        result.put("hdop", value.hdop);
        result.put("vdop", value.vdop);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetSatelliteSystemStatus(@Nullable ThingActions actions, int satelliteSystem) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetSatelliteSystemStatus(satelliteSystem);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Satellite Status")
    public @ActionOutput(name = "elevation", type="int")
           @ActionOutput(name = "azimuth", type="int")
           @ActionOutput(name = "snr", type="int")
           Map<String, Object> brickletGPSV2GetSatelliteStatus(
            @ActionInput(name = "satelliteSystem") int satelliteSystem,
            @ActionInput(name = "satelliteNumber") int satelliteNumber) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.SatelliteStatus value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getSatelliteStatus(satelliteSystem, satelliteNumber);
        result.put("elevation", value.elevation);
        result.put("azimuth", value.azimuth);
        result.put("snr", value.snr);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetSatelliteStatus(@Nullable ThingActions actions, int satelliteSystem, int satelliteNumber) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetSatelliteStatus(satelliteSystem, satelliteNumber);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Restart")
    public void brickletGPSV2Restart(
            @ActionInput(name = "restartType") int restartType) throws TinkerforgeException {
        ((BrickletGPSV2Wrapper)this.handler.getDevice()).restart(restartType);
    }

    public static void brickletGPSV2Restart(@Nullable ThingActions actions, int restartType) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            ((BrickletGPSV2Actions) actions).brickletGPSV2Restart(restartType);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Fix LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletGPSV2GetFixLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getFixLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetFixLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetFixLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get SBAS Config")
    public @ActionOutput(name = "sbasConfig", type="int")
           Map<String, Object> brickletGPSV2GetSBASConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getSBASConfig();
        result.put("sbasConfig", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetSBASConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetSBASConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletGPSV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletGPSV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletGPSV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletGPSV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletGPSV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.SPITFPErrorCount value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletGPSV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletGPSV2.Identity value = ((BrickletGPSV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletGPSV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletGPSV2Actions) {
            return ((BrickletGPSV2Actions) actions).brickletGPSV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletGPSV2Actions class.");
        }
    }
}
