package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletHallEffect;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHallEffectActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean")
           Map<String, Object> brickletHallEffectGetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletHallEffectWrapper)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletHallEffectGetEdgeCount(
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletHallEffectWrapper)this.handler.getDevice()).getEdgeCount(resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetEdgeCount(@Nullable ThingActions actions, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetEdgeCount(resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> brickletHallEffectGetEdgeCountConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffect.EdgeCountConfig value = ((BrickletHallEffectWrapper)this.handler.getDevice()).getEdgeCountConfig();
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetEdgeCountConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetEdgeCountConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletHallEffectGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffect.Identity value = ((BrickletHallEffectWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletHallEffectGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectActions) {
            return ((BrickletHallEffectActions) actions).brickletHallEffectGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectActions class.");
        }
    }
}
