package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletHallEffectV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHallEffectV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Magnetic Flux Density")
    public @ActionOutput(name = "magneticFluxDensity", type="int")
           Map<String, Object> brickletHallEffectV2GetMagneticFluxDensity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getMagneticFluxDensity();
        result.put("magneticFluxDensity", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetMagneticFluxDensity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetMagneticFluxDensity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletHallEffectV2GetCounter(
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getCounter(resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetCounter(@Nullable ThingActions actions, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetCounter(resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Config")
    public @ActionOutput(name = "highThreshold", type="int")
           @ActionOutput(name = "lowThreshold", type="int")
           @ActionOutput(name = "debounce", type="long")
           Map<String, Object> brickletHallEffectV2GetCounterConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffectV2.CounterConfig value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getCounterConfig();
        result.put("highThreshold", value.highThreshold);
        result.put("lowThreshold", value.lowThreshold);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetCounterConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetCounterConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletHallEffectV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletHallEffectV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletHallEffectV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletHallEffectV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletHallEffectV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffectV2.SPITFPErrorCount value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletHallEffectV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffectV2.Identity value = ((BrickletHallEffectV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletHallEffectV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).brickletHallEffectV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }
}
