package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIO16;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO16Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Port")
    public @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIO16GetPort(
            @ActionInput(name = "port") char port) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIO16Wrapper)this.handler.getDevice()).getPort(port);
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIO16GetPort(@Nullable ThingActions actions, char port) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetPort(port);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Port Configuration")
    public @ActionOutput(name = "directionMask", type="int")
           @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIO16GetPortConfiguration(
            @ActionInput(name = "port") char port) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.PortConfiguration value = ((BrickletIO16Wrapper)this.handler.getDevice()).getPortConfiguration(port);
        result.put("directionMask", value.directionMask);
        result.put("valueMask", value.valueMask);
        return result;
    }

    public static Map<String, Object> brickletIO16GetPortConfiguration(@Nullable ThingActions actions, char port) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetPortConfiguration(port);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIO16GetEdgeCount(
            @ActionInput(name = "pin") int pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO16Wrapper)this.handler.getDevice()).getEdgeCount(pin, resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIO16GetEdgeCount(@Nullable ThingActions actions, int pin, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetEdgeCount(pin, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Port Monoflop")
    public @ActionOutput(name = "value", type="int")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIO16GetPortMonoflop(
            @ActionInput(name = "port") char port,
            @ActionInput(name = "pin") int pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.PortMonoflop value = ((BrickletIO16Wrapper)this.handler.getDevice()).getPortMonoflop(port, pin);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIO16GetPortMonoflop(@Nullable ThingActions actions, char port, int pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetPortMonoflop(port, pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> brickletIO16GetEdgeCountConfig(
            @ActionInput(name = "pin") int pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.EdgeCountConfig value = ((BrickletIO16Wrapper)this.handler.getDevice()).getEdgeCountConfig(pin);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIO16GetEdgeCountConfig(@Nullable ThingActions actions, int pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetEdgeCountConfig(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIO16GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.Identity value = ((BrickletIO16Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIO16GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).brickletIO16GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }
}
