package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIO16V2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO16V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> brickletIO16V2GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "direction", type="char")
           @ActionOutput(name = "value", type="boolean")
           Map<String, Object> brickletIO16V2GetConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16V2.Configuration value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getConfiguration(channel);
        result.put("direction", value.direction);
        result.put("value", value.value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIO16V2GetEdgeCount(
            @ActionInput(name = "channel") int channel,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getEdgeCount(channel, resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetEdgeCount(@Nullable ThingActions actions, int channel, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetEdgeCount(channel, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIO16V2GetMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16V2.Monoflop value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getMonoflop(channel);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetMonoflop(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Configuration")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> brickletIO16V2GetEdgeCountConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16V2.EdgeCountConfiguration value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getEdgeCountConfiguration(channel);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetEdgeCountConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetEdgeCountConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIO16V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIO16V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIO16V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIO16V2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIO16V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16V2.SPITFPErrorCount value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIO16V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16V2.Identity value = ((BrickletIO16V2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIO16V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO16V2Actions) {
            return ((BrickletIO16V2Actions) actions).brickletIO16V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16V2Actions class.");
        }
    }
}
