package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialAnalogOut;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickletIndustrialAnalogOutIsEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletIndustrialAnalogOutWrapper)this.handler.getDevice()).isEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutIsEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletIndustrialAnalogOutGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialAnalogOutWrapper)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletIndustrialAnalogOutGetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialAnalogOutWrapper)this.handler.getDevice()).getCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "voltageRange", type="int")
           @ActionOutput(name = "currentRange", type="int")
           Map<String, Object> brickletIndustrialAnalogOutGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialAnalogOut.Configuration value = ((BrickletIndustrialAnalogOutWrapper)this.handler.getDevice()).getConfiguration();
        result.put("voltageRange", value.voltageRange);
        result.put("currentRange", value.currentRange);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialAnalogOutGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialAnalogOut.Identity value = ((BrickletIndustrialAnalogOutWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialAnalogOutGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).brickletIndustrialAnalogOutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }
}
