package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialDigitalOut4V2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDigitalOut4V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalOut4V2.Monoflop value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getMonoflop(channel);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetMonoflop(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getChannelLEDConfig(channel);
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Set PWM Configuration")
    public void brickletIndustrialDigitalOut4V2SetPWMConfiguration(
            @ActionInput(name = "channel") int channel,
            @ActionInput(name = "frequency") long frequency,
            @ActionInput(name = "dutyCycle") int dutyCycle) throws TinkerforgeException {
        ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).setPWMConfiguration(channel, frequency, dutyCycle);
    }

    public static void brickletIndustrialDigitalOut4V2SetPWMConfiguration(@Nullable ThingActions actions, int channel, long frequency, int dutyCycle) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2SetPWMConfiguration(channel, frequency, dutyCycle);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get PWM Configuration")
    public @ActionOutput(name = "frequency", type="long")
           @ActionOutput(name = "dutyCycle", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetPWMConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalOut4V2.PWMConfiguration value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getPWMConfiguration(channel);
        result.put("frequency", value.frequency);
        result.put("dutyCycle", value.dutyCycle);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetPWMConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetPWMConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIndustrialDigitalOut4V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalOut4V2.SPITFPErrorCount value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDigitalOut4V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalOut4V2.Identity value = ((BrickletIndustrialDigitalOut4V2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalOut4V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalOut4V2Actions) {
            return ((BrickletIndustrialDigitalOut4V2Actions) actions).brickletIndustrialDigitalOut4V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalOut4V2Actions class.");
        }
    }
}
