package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialQuadRelay;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialQuadRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIndustrialQuadRelayGetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getValue();
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="int")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIndustrialQuadRelayGetMonoflop(
            @ActionInput(name = "pin") int pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelay.Monoflop value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getMonoflop(pin);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetMonoflop(@Nullable ThingActions actions, int pin) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetMonoflop(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialQuadRelayGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelay.Identity value = ((BrickletIndustrialQuadRelayWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayActions) {
            return ((BrickletIndustrialQuadRelayActions) actions).brickletIndustrialQuadRelayGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayActions class.");
        }
    }
}
