package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialQuadRelayV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialQuadRelayV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> brickletIndustrialQuadRelayV2GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialQuadRelayV2GetChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getChannelLEDConfig(channel);
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIndustrialQuadRelayV2GetMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelayV2.Monoflop value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getMonoflop(channel);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetMonoflop(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIndustrialQuadRelayV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialQuadRelayV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialQuadRelayV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIndustrialQuadRelayV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIndustrialQuadRelayV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelayV2.SPITFPErrorCount value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialQuadRelayV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialQuadRelayV2.Identity value = ((BrickletIndustrialQuadRelayV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialQuadRelayV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialQuadRelayV2Actions) {
            return ((BrickletIndustrialQuadRelayV2Actions) actions).brickletIndustrialQuadRelayV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialQuadRelayV2Actions class.");
        }
    }
}
