package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletJoystick;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletJoystickActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Position")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           Map<String, Object> brickletJoystickGetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystick.Position value = ((BrickletJoystickWrapper)this.handler.getDevice()).getPosition();
        result.put("x", value.x);
        result.put("y", value.y);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> brickletJoystickIsPressed(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletJoystickWrapper)this.handler.getDevice()).isPressed();
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickIsPressed(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickIsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           Map<String, Object> brickletJoystickGetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystick.AnalogValue value = ((BrickletJoystickWrapper)this.handler.getDevice()).getAnalogValue();
        result.put("x", value.x);
        result.put("y", value.y);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletJoystickGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystick.Identity value = ((BrickletJoystickWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletJoystickGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickActions) {
            return ((BrickletJoystickActions) actions).brickletJoystickGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickActions class.");
        }
    }
}
