package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletJoystickV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletJoystickV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Position")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           Map<String, Object> brickletJoystickV2GetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystickV2.Position value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getPosition();
        result.put("x", value.x);
        result.put("y", value.y);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Is Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> brickletJoystickV2IsPressed(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).isPressed();
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2IsPressed(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2IsPressed();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletJoystickV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletJoystickV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletJoystickV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletJoystickV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletJoystickV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystickV2.SPITFPErrorCount value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletJoystickV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletJoystickV2.Identity value = ((BrickletJoystickV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletJoystickV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletJoystickV2Actions) {
            return ((BrickletJoystickV2Actions) actions).brickletJoystickV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletJoystickV2Actions class.");
        }
    }
}
