package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletLEDStrip;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set RGB Values")
    public void brickletLEDStripSetRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length,
            @ActionInput(name = "r") int[] r,
            @ActionInput(name = "g") int[] g,
            @ActionInput(name = "b") int[] b) throws TinkerforgeException {
        ((BrickletLEDStripWrapper)this.handler.getDevice()).setRGBValues(index, length, r, g, b);
    }

    public static void brickletLEDStripSetRGBValues(@Nullable ThingActions actions, int index, int length, int[] r, int[] g, int[] b) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).brickletLEDStripSetRGBValues(index, length, r, g, b);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGB Values")
    public @ActionOutput(name = "r", type="int[]")
           @ActionOutput(name = "g", type="int[]")
           @ActionOutput(name = "b", type="int[]")
           Map<String, Object> brickletLEDStripGetRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLEDStrip.RGBValues value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getRGBValues(index, length);
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBValues(@Nullable ThingActions actions, int index, int length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetRGBValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> brickletLEDStripGetFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getFrameDuration();
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetFrameDuration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletLEDStripGetSupplyVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getSupplyVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetSupplyVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetSupplyVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type="long")
           Map<String, Object> brickletLEDStripGetClockFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getClockFrequency();
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetClockFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetClockFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type="int")
           Map<String, Object> brickletLEDStripGetChipType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getChipType();
        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChipType(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetChipType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Set RGBW Values")
    public void brickletLEDStripSetRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length,
            @ActionInput(name = "r") int[] r,
            @ActionInput(name = "g") int[] g,
            @ActionInput(name = "b") int[] b,
            @ActionInput(name = "w") int[] w) throws TinkerforgeException {
        ((BrickletLEDStripWrapper)this.handler.getDevice()).setRGBWValues(index, length, r, g, b, w);
    }

    public static void brickletLEDStripSetRGBWValues(@Nullable ThingActions actions, int index, int length, int[] r, int[] g, int[] b, int[] w) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).brickletLEDStripSetRGBWValues(index, length, r, g, b, w);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGBW Values")
    public @ActionOutput(name = "r", type="int[]")
           @ActionOutput(name = "g", type="int[]")
           @ActionOutput(name = "b", type="int[]")
           @ActionOutput(name = "w", type="int[]")
           Map<String, Object> brickletLEDStripGetRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLEDStrip.RGBWValues value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getRGBWValues(index, length);
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("w", value.w);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetRGBWValues(@Nullable ThingActions actions, int index, int length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetRGBWValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type="int")
           Map<String, Object> brickletLEDStripGetChannelMapping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getChannelMapping();
        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetChannelMapping(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetChannelMapping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLEDStripGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLEDStrip.Identity value = ((BrickletLEDStripWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLEDStripGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).brickletLEDStripGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }
}
