package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletLoadCellV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLoadCellV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Weight")
    public @ActionOutput(name = "weight", type="int")
           Map<String, Object> brickletLoadCellV2GetWeight(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getWeight();
        result.put("weight", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetWeight(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetWeight();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Info LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletLoadCellV2GetInfoLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getInfoLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetInfoLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetInfoLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Tare")
    public void brickletLoadCellV2Tare(
            ) throws TinkerforgeException {
        ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).tare();
    }

    public static void brickletLoadCellV2Tare(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2Tare();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="int")
           Map<String, Object> brickletLoadCellV2GetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getMovingAverage();
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "rate", type="int")
           @ActionOutput(name = "gain", type="int")
           Map<String, Object> brickletLoadCellV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLoadCellV2.Configuration value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getConfiguration();
        result.put("rate", value.rate);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletLoadCellV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletLoadCellV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletLoadCellV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletLoadCellV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLoadCellV2.SPITFPErrorCount value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLoadCellV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLoadCellV2.Identity value = ((BrickletLoadCellV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLoadCellV2Actions) {
            return ((BrickletLoadCellV2Actions) actions).brickletLoadCellV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLoadCellV2Actions class.");
        }
    }
}
