package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMotionDetector;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotionDetectorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Motion Detected")
    public @ActionOutput(name = "motion", type="int")
           Map<String, Object> brickletMotionDetectorGetMotionDetected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorWrapper)this.handler.getDevice()).getMotionDetected();
        result.put("motion", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorGetMotionDetected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorActions) {
            return ((BrickletMotionDetectorActions) actions).brickletMotionDetectorGetMotionDetected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMotionDetectorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotionDetector.Identity value = ((BrickletMotionDetectorWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorActions) {
            return ((BrickletMotionDetectorActions) actions).brickletMotionDetectorGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorActions class.");
        }
    }
}
