package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMotorizedLinearPoti;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotorizedLinearPotiActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Position")
    public @ActionOutput(name = "position", type="int")
           Map<String, Object> brickletMotorizedLinearPotiGetPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getPosition();
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Motor Position")
    public @ActionOutput(name = "position", type="int")
           @ActionOutput(name = "driveMode", type="int")
           @ActionOutput(name = "holdPosition", type="boolean")
           @ActionOutput(name = "positionReached", type="boolean")
           Map<String, Object> brickletMotorizedLinearPotiGetMotorPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotorizedLinearPoti.MotorPosition value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getMotorPosition();
        result.put("position", value.position);
        result.put("driveMode", value.driveMode);
        result.put("holdPosition", value.holdPosition);
        result.put("positionReached", value.positionReached);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetMotorPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetMotorPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletMotorizedLinearPotiReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletMotorizedLinearPotiGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletMotorizedLinearPotiGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletMotorizedLinearPotiGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletMotorizedLinearPotiGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotorizedLinearPoti.SPITFPErrorCount value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMotorizedLinearPotiGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotorizedLinearPoti.Identity value = ((BrickletMotorizedLinearPotiWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotorizedLinearPotiGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotorizedLinearPotiActions) {
            return ((BrickletMotorizedLinearPotiActions) actions).brickletMotorizedLinearPotiGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotorizedLinearPotiActions class.");
        }
    }
}
