package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMultiTouchV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMultiTouchV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Touch State")
    public @ActionOutput(name = "state", type="boolean[]")
           Map<String, Object> brickletMultiTouchV2GetTouchState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getTouchState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetTouchState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetTouchState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Recalibrate")
    public void brickletMultiTouchV2Recalibrate(
            ) throws TinkerforgeException {
        ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).recalibrate();
    }

    public static void brickletMultiTouchV2Recalibrate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2Recalibrate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Config")
    public @ActionOutput(name = "enabledElectrodes", type="boolean[]")
           Map<String, Object> brickletMultiTouchV2GetElectrodeConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getElectrodeConfig();
        result.put("enabledElectrodes", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetElectrodeConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetElectrodeConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Sensitivity")
    public @ActionOutput(name = "sensitivity", type="int")
           Map<String, Object> brickletMultiTouchV2GetElectrodeSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getElectrodeSensitivity();
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetElectrodeSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetElectrodeSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Touch LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletMultiTouchV2GetTouchLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getTouchLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetTouchLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetTouchLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletMultiTouchV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletMultiTouchV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletMultiTouchV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletMultiTouchV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletMultiTouchV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMultiTouchV2.SPITFPErrorCount value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMultiTouchV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMultiTouchV2.Identity value = ((BrickletMultiTouchV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).brickletMultiTouchV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }
}
