package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletOutdoorWeather;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOutdoorWeatherActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Station Identifiers")
    public @ActionOutput(name = "identifiers", type="int[]")
           Map<String, Object> brickletOutdoorWeatherGetStationIdentifiers(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getStationIdentifiers();
        result.put("identifiers", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetStationIdentifiers(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetStationIdentifiers();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Sensor Identifiers")
    public @ActionOutput(name = "identifiers", type="int[]")
           Map<String, Object> brickletOutdoorWeatherGetSensorIdentifiers(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getSensorIdentifiers();
        result.put("identifiers", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetSensorIdentifiers(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetSensorIdentifiers();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Station Data")
    public @ActionOutput(name = "temperature", type="int")
           @ActionOutput(name = "humidity", type="int")
           @ActionOutput(name = "windSpeed", type="long")
           @ActionOutput(name = "gustSpeed", type="long")
           @ActionOutput(name = "rain", type="long")
           @ActionOutput(name = "windDirection", type="int")
           @ActionOutput(name = "batteryLow", type="boolean")
           @ActionOutput(name = "lastChange", type="int")
           Map<String, Object> brickletOutdoorWeatherGetStationData(
            @ActionInput(name = "identifier") int identifier) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOutdoorWeather.StationData value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getStationData(identifier);
        result.put("temperature", value.temperature);
        result.put("humidity", value.humidity);
        result.put("windSpeed", value.windSpeed);
        result.put("gustSpeed", value.gustSpeed);
        result.put("rain", value.rain);
        result.put("windDirection", value.windDirection);
        result.put("batteryLow", value.batteryLow);
        result.put("lastChange", value.lastChange);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetStationData(@Nullable ThingActions actions, int identifier) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetStationData(identifier);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Sensor Data")
    public @ActionOutput(name = "temperature", type="int")
           @ActionOutput(name = "humidity", type="int")
           @ActionOutput(name = "lastChange", type="int")
           Map<String, Object> brickletOutdoorWeatherGetSensorData(
            @ActionInput(name = "identifier") int identifier) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOutdoorWeather.SensorData value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getSensorData(identifier);
        result.put("temperature", value.temperature);
        result.put("humidity", value.humidity);
        result.put("lastChange", value.lastChange);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetSensorData(@Nullable ThingActions actions, int identifier) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetSensorData(identifier);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletOutdoorWeatherReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletOutdoorWeatherGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOutdoorWeatherGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletOutdoorWeatherGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletOutdoorWeatherGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOutdoorWeather.SPITFPErrorCount value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOutdoorWeatherGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOutdoorWeather.Identity value = ((BrickletOutdoorWeatherWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOutdoorWeatherGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOutdoorWeatherActions) {
            return ((BrickletOutdoorWeatherActions) actions).brickletOutdoorWeatherGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOutdoorWeatherActions class.");
        }
    }
}
