package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletParticulateMatter;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletParticulateMatterActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get PM Concentration")
    public @ActionOutput(name = "pm10", type="int")
           @ActionOutput(name = "pm25", type="int")
           @ActionOutput(name = "pm100", type="int")
           Map<String, Object> brickletParticulateMatterGetPMConcentration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.PMConcentration value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getPMConcentration();
        result.put("pm10", value.pm10);
        result.put("pm25", value.pm25);
        result.put("pm100", value.pm100);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetPMConcentration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetPMConcentration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get PM Count")
    public @ActionOutput(name = "greater03um", type="int")
           @ActionOutput(name = "greater05um", type="int")
           @ActionOutput(name = "greater10um", type="int")
           @ActionOutput(name = "greater25um", type="int")
           @ActionOutput(name = "greater50um", type="int")
           @ActionOutput(name = "greater100um", type="int")
           Map<String, Object> brickletParticulateMatterGetPMCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.PMCount value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getPMCount();
        result.put("greater03um", value.greater03um);
        result.put("greater05um", value.greater05um);
        result.put("greater10um", value.greater10um);
        result.put("greater25um", value.greater25um);
        result.put("greater50um", value.greater50um);
        result.put("greater100um", value.greater100um);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetPMCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetPMCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Sensor Info")
    public @ActionOutput(name = "sensorVersion", type="int")
           @ActionOutput(name = "lastErrorCode", type="int")
           @ActionOutput(name = "framingErrorCount", type="int")
           @ActionOutput(name = "checksumErrorCount", type="int")
           Map<String, Object> brickletParticulateMatterGetSensorInfo(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.SensorInfo value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getSensorInfo();
        result.put("sensorVersion", value.sensorVersion);
        result.put("lastErrorCode", value.lastErrorCode);
        result.put("framingErrorCount", value.framingErrorCount);
        result.put("checksumErrorCount", value.checksumErrorCount);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetSensorInfo(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetSensorInfo();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletParticulateMatterReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletParticulateMatterGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletParticulateMatterGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletParticulateMatterGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletParticulateMatterGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.SPITFPErrorCount value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletParticulateMatterGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.Identity value = ((BrickletParticulateMatterWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletParticulateMatterGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).brickletParticulateMatterGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }
}
