package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletPiezoSpeakerV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPiezoSpeakerV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Beep")
    public void brickletPiezoSpeakerV2SetBeep(
            @ActionInput(name = "frequency") int frequency,
            @ActionInput(name = "volume") int volume,
            @ActionInput(name = "duration") long duration) throws TinkerforgeException {
        ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).setBeep(frequency, volume, duration);
    }

    public static void brickletPiezoSpeakerV2SetBeep(@Nullable ThingActions actions, int frequency, int volume, long duration) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2SetBeep(frequency, volume, duration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Beep")
    public @ActionOutput(name = "frequency", type="int")
           @ActionOutput(name = "volume", type="int")
           @ActionOutput(name = "duration", type="long")
           @ActionOutput(name = "durationRemaining", type="long")
           Map<String, Object> brickletPiezoSpeakerV2GetBeep(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoSpeakerV2.Beep value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getBeep();
        result.put("frequency", value.frequency);
        result.put("volume", value.volume);
        result.put("duration", value.duration);
        result.put("durationRemaining", value.durationRemaining);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetBeep(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetBeep();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Set Alarm")
    public void brickletPiezoSpeakerV2SetAlarm(
            @ActionInput(name = "startFrequency") int startFrequency,
            @ActionInput(name = "endFrequency") int endFrequency,
            @ActionInput(name = "stepSize") int stepSize,
            @ActionInput(name = "stepDelay") int stepDelay,
            @ActionInput(name = "volume") int volume,
            @ActionInput(name = "duration") long duration) throws TinkerforgeException {
        ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).setAlarm(startFrequency, endFrequency, stepSize, stepDelay, volume, duration);
    }

    public static void brickletPiezoSpeakerV2SetAlarm(@Nullable ThingActions actions, int startFrequency, int endFrequency, int stepSize, int stepDelay, int volume, long duration) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2SetAlarm(startFrequency, endFrequency, stepSize, stepDelay, volume, duration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Alarm")
    public @ActionOutput(name = "startFrequency", type="int")
           @ActionOutput(name = "endFrequency", type="int")
           @ActionOutput(name = "stepSize", type="int")
           @ActionOutput(name = "stepDelay", type="int")
           @ActionOutput(name = "volume", type="int")
           @ActionOutput(name = "duration", type="long")
           @ActionOutput(name = "durationRemaining", type="long")
           @ActionOutput(name = "currentFrequency", type="int")
           Map<String, Object> brickletPiezoSpeakerV2GetAlarm(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoSpeakerV2.Alarm value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getAlarm();
        result.put("startFrequency", value.startFrequency);
        result.put("endFrequency", value.endFrequency);
        result.put("stepSize", value.stepSize);
        result.put("stepDelay", value.stepDelay);
        result.put("volume", value.volume);
        result.put("duration", value.duration);
        result.put("durationRemaining", value.durationRemaining);
        result.put("currentFrequency", value.currentFrequency);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetAlarm(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetAlarm();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Update Volume")
    public void brickletPiezoSpeakerV2UpdateVolume(
            @ActionInput(name = "volume") int volume) throws TinkerforgeException {
        ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).updateVolume(volume);
    }

    public static void brickletPiezoSpeakerV2UpdateVolume(@Nullable ThingActions actions, int volume) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2UpdateVolume(volume);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Update Frequency")
    public void brickletPiezoSpeakerV2UpdateFrequency(
            @ActionInput(name = "frequency") int frequency) throws TinkerforgeException {
        ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).updateFrequency(frequency);
    }

    public static void brickletPiezoSpeakerV2UpdateFrequency(@Nullable ThingActions actions, int frequency) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2UpdateFrequency(frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletPiezoSpeakerV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPiezoSpeakerV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletPiezoSpeakerV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletPiezoSpeakerV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletPiezoSpeakerV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoSpeakerV2.SPITFPErrorCount value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPiezoSpeakerV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoSpeakerV2.Identity value = ((BrickletPiezoSpeakerV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerV2Actions) {
            return ((BrickletPiezoSpeakerV2Actions) actions).brickletPiezoSpeakerV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerV2Actions class.");
        }
    }
}
