package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRGBLED;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get RGB Value")
    public @ActionOutput(name = "r", type="int")
           @ActionOutput(name = "g", type="int")
           @ActionOutput(name = "b", type="int")
           Map<String, Object> brickletRGBLEDGetRGBValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLED.RGBValue value = ((BrickletRGBLEDWrapper)this.handler.getDevice()).getRGBValue();
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDGetRGBValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDActions) {
            return ((BrickletRGBLEDActions) actions).brickletRGBLEDGetRGBValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRGBLEDGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLED.Identity value = ((BrickletRGBLEDWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDActions) {
            return ((BrickletRGBLEDActions) actions).brickletRGBLEDGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDActions class.");
        }
    }
}
