package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRGBLEDV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get RGB Value")
    public @ActionOutput(name = "r", type="int")
           @ActionOutput(name = "g", type="int")
           @ActionOutput(name = "b", type="int")
           Map<String, Object> brickletRGBLEDV2GetRGBValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDV2.RGBValue value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getRGBValue();
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetRGBValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetRGBValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRGBLEDV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRGBLEDV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRGBLEDV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRGBLEDV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRGBLEDV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDV2.SPITFPErrorCount value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRGBLEDV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDV2.Identity value = ((BrickletRGBLEDV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRGBLEDV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDV2Actions) {
            return ((BrickletRGBLEDV2Actions) actions).brickletRGBLEDV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDV2Actions class.");
        }
    }
}
