package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRealTimeClockV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRealTimeClockV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "year", type="int")
           @ActionOutput(name = "month", type="int")
           @ActionOutput(name = "day", type="int")
           @ActionOutput(name = "hour", type="int")
           @ActionOutput(name = "minute", type="int")
           @ActionOutput(name = "second", type="int")
           @ActionOutput(name = "centisecond", type="int")
           @ActionOutput(name = "weekday", type="int")
           @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockV2GetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClockV2.DateTime value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getDateTime();
        result.put("year", value.year);
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("centisecond", value.centisecond);
        result.put("weekday", value.weekday);
        result.put("timestamp", value.timestamp);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetDateTime();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Timestamp")
    public @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockV2GetTimestamp(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getTimestamp();
        result.put("timestamp", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetTimestamp(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetTimestamp();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Offset")
    public @ActionOutput(name = "offset", type="int")
           Map<String, Object> brickletRealTimeClockV2GetOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getOffset();
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetOffset();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRealTimeClockV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRealTimeClockV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRealTimeClockV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRealTimeClockV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRealTimeClockV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClockV2.SPITFPErrorCount value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRealTimeClockV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClockV2.Identity value = ((BrickletRealTimeClockV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockV2Actions) {
            return ((BrickletRealTimeClockV2Actions) actions).brickletRealTimeClockV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockV2Actions class.");
        }
    }
}
