package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletSegmentDisplay4x7V2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSegmentDisplay4x7V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Segments")
    public @ActionOutput(name = "digit0", type="boolean[]")
           @ActionOutput(name = "digit1", type="boolean[]")
           @ActionOutput(name = "digit2", type="boolean[]")
           @ActionOutput(name = "digit3", type="boolean[]")
           @ActionOutput(name = "colon", type="boolean[]")
           @ActionOutput(name = "tick", type="boolean")
           Map<String, Object> brickletSegmentDisplay4x7V2GetSegments(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7V2.Segments value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getSegments();
        result.put("digit0", value.digit0);
        result.put("digit1", value.digit1);
        result.put("digit2", value.digit2);
        result.put("digit3", value.digit3);
        result.put("colon", value.colon);
        result.put("tick", value.tick);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSegments(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetSegments();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Brightness")
    public @ActionOutput(name = "brightness", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetBrightness(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getBrightness();
        result.put("brightness", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetBrightness(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetBrightness();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Selected Segment")
    public @ActionOutput(name = "value", type="boolean")
           Map<String, Object> brickletSegmentDisplay4x7V2GetSelectedSegment(
            @ActionInput(name = "segment") int segment) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getSelectedSegment(segment);
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSelectedSegment(@Nullable ThingActions actions, int segment) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetSelectedSegment(segment);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Start Counter")
    public void brickletSegmentDisplay4x7V2StartCounter(
            @ActionInput(name = "valueFrom") int valueFrom,
            @ActionInput(name = "valueTo") int valueTo,
            @ActionInput(name = "increment") int increment,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).startCounter(valueFrom, valueTo, increment, length);
    }

    public static void brickletSegmentDisplay4x7V2StartCounter(@Nullable ThingActions actions, int valueFrom, int valueTo, int increment, long length) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2StartCounter(valueFrom, valueTo, increment, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetCounterValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getCounterValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetCounterValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetCounterValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletSegmentDisplay4x7V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletSegmentDisplay4x7V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7V2.SPITFPErrorCount value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletSegmentDisplay4x7V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7V2.Identity value = ((BrickletSegmentDisplay4x7V2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).brickletSegmentDisplay4x7V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }
}
