package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletSolidStateRelayV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSolidStateRelayV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type="boolean")
           Map<String, Object> brickletSolidStateRelayV2GetState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletSolidStateRelayV2GetMonoflop(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSolidStateRelayV2.Monoflop value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getMonoflop();
        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetMonoflop(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetMonoflop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletSolidStateRelayV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletSolidStateRelayV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletSolidStateRelayV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletSolidStateRelayV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletSolidStateRelayV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSolidStateRelayV2.SPITFPErrorCount value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletSolidStateRelayV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSolidStateRelayV2.Identity value = ((BrickletSolidStateRelayV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSolidStateRelayV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayV2Actions) {
            return ((BrickletSolidStateRelayV2Actions) actions).brickletSolidStateRelayV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayV2Actions class.");
        }
    }
}
