package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletTemperatureIRV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureIRV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Ambient Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureIRV2GetAmbientTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getAmbientTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetAmbientTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetAmbientTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Object Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureIRV2GetObjectTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getObjectTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetObjectTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetObjectTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Emissivity")
    public @ActionOutput(name = "emissivity", type="int")
           Map<String, Object> brickletTemperatureIRV2GetEmissivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getEmissivity();
        result.put("emissivity", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetEmissivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetEmissivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletTemperatureIRV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletTemperatureIRV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletTemperatureIRV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletTemperatureIRV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletTemperatureIRV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletTemperatureIRV2.SPITFPErrorCount value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletTemperatureIRV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletTemperatureIRV2.Identity value = ((BrickletTemperatureIRV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletTemperatureIRV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureIRV2Actions) {
            return ((BrickletTemperatureIRV2Actions) actions).brickletTemperatureIRV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureIRV2Actions class.");
        }
    }
}
