package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletThermocouple;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletThermocoupleGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleWrapper)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "thermocoupleType", type="int")
           @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletThermocoupleGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocouple.Configuration value = ((BrickletThermocoupleWrapper)this.handler.getDevice()).getConfiguration();
        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type="boolean")
           @ActionOutput(name = "openCircuit", type="boolean")
           Map<String, Object> brickletThermocoupleGetErrorState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocouple.ErrorState value = ((BrickletThermocoupleWrapper)this.handler.getDevice()).getErrorState();
        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetErrorState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetErrorState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletThermocoupleGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocouple.Identity value = ((BrickletThermocoupleWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleActions) {
            return ((BrickletThermocoupleActions) actions).brickletThermocoupleGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleActions class.");
        }
    }
}
