package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletThermocoupleV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletThermocoupleV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "thermocoupleType", type="int")
           @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletThermocoupleV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.Configuration value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getConfiguration();
        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type="boolean")
           @ActionOutput(name = "openCircuit", type="boolean")
           Map<String, Object> brickletThermocoupleV2GetErrorState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.ErrorState value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getErrorState();
        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetErrorState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetErrorState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletThermocoupleV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletThermocoupleV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletThermocoupleV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.SPITFPErrorCount value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletThermocoupleV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.Identity value = ((BrickletThermocoupleV2Wrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletThermocoupleV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).brickletThermocoupleV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }
}
