package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletXMC1400Breakout;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletXMC1400BreakoutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletXMC1400BreakoutReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletXMC1400BreakoutGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletXMC1400BreakoutGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletXMC1400BreakoutGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletXMC1400BreakoutGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletXMC1400Breakout.SPITFPErrorCount value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletXMC1400BreakoutGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletXMC1400Breakout.Identity value = ((BrickletXMC1400BreakoutWrapper)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletXMC1400BreakoutGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletXMC1400BreakoutActions) {
            return ((BrickletXMC1400BreakoutActions) actions).brickletXMC1400BreakoutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletXMC1400BreakoutActions class.");
        }
    }
}
